#!/usr/bin/perl
# parse linux-sound.org html files
# wget -m http://linux-sound.org/

#-#-#-#-#-#-#-# CONFIG #-#-#-#-#-#-#-#

#  1: indexer - Headings / topic
#  2: indexer - file data
#  4: indexer+createcat - Categories
#  8: dir-list: topic parser
# 16: build: warn dup files during build.
# 32: build: newlines (list output files)
# 64: report: display dup file list when $renamedups=1
#128: build: dump cats/topic during building
#
my $debug=128|8|16;

# if($renamedups): duplicate enties won't be appended to existing files,
# but new filenames  XXX-1.txt, XXX-2.txt are used instead.
# this disables the dup-report at the end (re-enable with $debug|=64;)
my $renamedups = 1;

$debug=0;

my $dirname="/home/rgareus/tmp/lao/linux-sound.org/";
my $dstdir="/tmp/wiki/";

# system("rm -rf $dstdir");
mkdir $dstdir;
mkdir $dstdir."/all";
mkdir $dstdir."/categories";

#-#-#-#-#-#-#-# END CONFIG #-#-#-#-#-#-#-#

#
# subroutines
#

my @software;
my %ctree = ();
my $catcount = 0;

sub index_file ($;$) {
	my $filename=shift;
	my $title=shift;
	my $topic=$title;
	my @cat;
	my $depth = 0;
	my $ignore = 0;

	for (my $i =0; $i<7; $i++) { $cat[$i]=""; }

	open(INPUT, "< $filename");
	# TODO: strip HTML comments..
	# reformat the html pages...
	foreach (<INPUT>) {
		$line=$_; 
		$line =~ s/\r//g;
		$str=$line;

		if ( $str =~ m/-->/) {
			$ignore=0;
			next;
		}
		if ( $str =~ m/^<!--/) {
			$ignore=1;
		}
		next if ($ignore == 1) ;

		if ( $str =~ m/^<\/h.?>$/) {
			$topic=$title;	 # FIXME reset default topic ?
		}
		if ( $str =~ m/^<\/b.?>$/) {
			$cat[$depth]="";
			for (my $i=$depth+1; $i<6; $i++) { $cat[$i]=""; }
		}

		if ( $str =~ m/^<\/ul>/) {
			$depth--;
		}

		if ( $str =~ m/^<ul>/) {
			$depth++;
		}

		if ( $str =~ m/^<li>\s*<b>([^<>]+)<\/b>\s*$/) {
			$cat[$depth]=$1;
			for (my $i=$depth+1; $i<6; $i++) { $cat[$i]=""; }
			if ($debug & 4) {
				print "!CATEGORY:!S:$depth:",$1,"\n";
			}
			next;
		}
		$str=$line;
		if ( $str =~ m/^<h([1-6])>([^<>]+)<\/h([1-6])>\s+$/) {
			# TODO assign different categories? (no - only h3's in source)
			$topic=$2;
			if ($debug & 4) {
				print "!CATEGORY:!",$1,"::",$topic,"\n";
			}
			next;
		}
		$str=$line;
		if ( $str =~ m/^<li>\s*<a href=[\"']?([^<>]+)[\"']?>([^<>]+)<\/a>(.*)$/i) {
			my %lss = ();
			$lss{'url'} = $1; 
			$lss{'name'} = $2; 
			$lss{'desc'} = $3;
			chomp $lss{'desc'};
			$lss{'desc'} =~ s/^\s//;
			$lss{'topic'} = $topic;
			$lss{'cats'} = [];
			my @cats;
			for (my $i=0; $i<7; $i++) { 
				next unless ($cat[$i]);
				push @{$lss{'cats'}} ,$cat[$i];
				if ($debug & 1) { print "#HEADING#",$i,"-",$cat[$i],"\n"; }
			}
			chomp %lss;
			if ($debug & 2) {
				print "-",$lss{'name'}," ::";
				print " ",$lss{'url'},"\n";
				print " ",$lss{'desc'},"\n";
			}
			push(@software,\%lss);
		} else {
			chomp($line);
			#print "UP:",$line,"\n";
		}
	}
}

sub makewiki ($){
	my $text = shift;
	$text =~ s/<\/?i>/\/\//g;
	$text =~ s/<\/?b>/\*\*/g;
	$text =~ s/<a\s+href\s*=\s*['"]*([^<>]+)['"]*\s*>([^<>]*)<\/a>/\[\[$1|$2\]\]/ig;
	## TODO: replace umlaut and UTF8 chars
	return($text)
}


sub wikiize_string ($){
	my $value = shift;
	$value =~ s/\s/_/g;
	$value =~ s/@/_at_/g;
	$value =~ s/&/_and/g;
	$value =~ s/[\/:\*\+,^]/_/g;
	$value =~ s/[!\.\(\)\*';\?\|]//g;
	$value =~ s/_+/_/g;
	$value =~ s/^_//;
	$value =~ s/_$//;
	return $value;
}


sub makemeta ($;$){
	my $key = shift;
	my $value = wikiize_string(shift);
	return ("~~META:$key=$value~~\n");
}

sub maketag ($){
	my $tag = wikiize_string(shift);
	return ("{{tag>",$tag,"}}\n");
}

sub makelink ($){
	my $tag = wikiize_string(shift);
	#return ("{{category>",$tag,"}}\n");
	return ("[[apps:categories:".$tag."]]");
}

%lstops= (
	"ardour-music" => "Music recorded with Ardour",
	"new" => "New Additions for Winter 2006-2007",
	"ldj" => "The Linux Digital DJ" );

sub parsetopic ($) {
	my $topic = shift;
	$topic =~ s/.html//;
	if($debug&8) { print "TOPIC '$topic' -> "; }
	if ($lstops{$topic}) { $topic=$lstops{$topic}; }
	if($debug&8) { print "$topic\n"; }
	return $topic;
}

sub createcat ($;$;@;@) {
	my $name=$_[0];
	my $parent=$_[1];
	my @sub0=$_[2];
	my @sub1=$_[3];
	my $fn=wikiize_string(lc($name));
	my $nl = 0;
	if ($debug & 4 ) { print "writing CAT: ".$fn.".txt\n"; }

	open( CAT , "> $dstdir/categories/".$fn.".txt");
	print CAT "===== ".$name." ===== \n\n";
	if ($parent)    { $nl=1; print CAT "  * ",maketag($parent); } 
	foreach (@sub0) { if($_) { $nl=2; print CAT "  - ",maketag($_); } }
	foreach (@sub1) { if($_) { $nl=3; print CAT "  - ",maketag($_); } } 

	if ($nl) { print CAT "\n"; }
	print CAT "{{topic>".$fn."}}\n";
	$catcount++;
	close(CAT);
}

#
# MAIN
#

opendir (DIR, $dirname);
while (defined($file = readdir(DIR))) {
	next unless $file =~ m/.html$/;	
	next if $file =~ m/index.html$/;	
	next if $file =~ m/toc.html$/;	
	next if $file =~ m/top.html$/;	
	next if $file =~ m/muse.html$/;	 # musings
	next if $file =~ m/one-page.html$/;	
	next if $file =~ m/new.html$/;	
	if (debug & ~16) { print "            +++ INDEXING FILE +++\n"; }
	print "parsing: $file\n";
	index_file($dirname."/".$file,parsetopic($file));
}

index_file($dirname."/new.html",parsetopic("new.html"));

print "###########################\n";
print "creating wiki files in: $dstdir/all/\n";

my $progres=0;
my $progtot=$#software;
my $dups = 0;
my $files = 0;
my $entries = 0;
my @duplicates;

if (!($debug&32)) { $|=1; }

foreach (@software) {
	$progres++;
	my %lss = %{$_};
	my $exists= 0;
	my $outbase=lc($lss{'name'});
	$outbase =~ s/[&:\*\+,^]/_/g;
	$outbase = wikiize_string($outbase);

	my $outfile =$outbase.".txt";
	$entries++;

	if ($renamedups) {
		my $i=0;
		while (-e $dstdir."/all/".$outfile) {
			$outfile=$outbase."-".$i.".txt";
			$i++;
			$exists=2;
		}
	}
	$outfile=$dstdir."/all/".$outfile;

	if ($debug&32) {
		printf (" [ %2.2f ] %s\n", 100.0*$progres/$progtot , $outfile);
	} else {
		printf ("\r [ %2.2f ] %s", 100.0*$progres/$progtot , substr($outfile,0,50));
		if (length $outfile > 50) { print ".."; }
		for (my $i= length $outfile; $i < 55; $i++) { print " "; }
		print "\r";
	}
	if (-e $outfile) {
		$exists=1;
	} else { $files++; } 

	if ($exists) {
		$dups ++;
		push @duplicates , $outfile;
		if (($debug&48) == 16) { print "\n"; }
		if ($debug&16) { 
			printf ("+ file exists: %s %s\n",($exists==2)?"->":"",$outfile); 
			}
	}
	open( SFH, ">> $outfile") or die "Can't write file $outfile: $!";
	print SFH "======",$lss{'name'},"======\n";
	print SFH "  * ",$lss{'url'},"\n";
	my $t1="-";
	if ($lss{'topic'}) {
		$t1=$lss{'topic'}; 
		print SFH makemeta("topic",$lss{'topic'});
		print SFH maketag($lss{'topic'});
		if ($debug &128) { print " ---TOPIC---",maketag($lss{'topic'});}
	} else {
		print " -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-= NO TOPIC!\n";
		print $outfile,"\n";
		print " -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-= NO TOPIC!\n";
	}
	if (!$ctree{$t1}){ %{$ctree{$t1}}=();}
	my $cnt=0;
	my $t2="-";
	foreach (@{$lss{'cats'}}) {
		$cnt++;
		print SFH maketag($_);
		if ($debug &128) { print " -----$cnt-----",maketag($_); }
		if ($cnt == 1) { 
			$t2=$_; if (!${$ctree{$t1}}{$t2}) { %{${$ctree{$t1}}{$t2}}=(); }
			print SFH makemeta("category",$_);
		}
		else {
			my $t3=$_; if (!${${$ctree{$t1}}{$t2}}{$t3}) { %{${${$ctree{$t1}}{$t2}}{$t3}}=(); }
#			print SFH makemeta("subcategory$cnt",$_);
		}
	}
	# TODO chomp ?
	print SFH makewiki($lss{'desc'})," \\\\\n";
	close(SFH);

  if (0) { # done by tagmege.pl 
	open( SFH, "+< $outfile");
	my @lines = <SFH>;
	my %seen = ();
	my $tags = "";
	truncate SFH, 0;
	seek SFH,0,0;
	print SFH "======",$lss{'name'},"======\n";
	foreach (sort grep { ! $seen{$_} ++ } @lines) {
		next if $_ =~ m/======/;
		print SFH $_;
	}
	close(SFH);
  }

}

if (($debug&48) == 16) { print "\n"; }

print "###########################\n";

print "creating indices: $dstdir/alltags.txt\n";
print "                : $dstdir/index.txt\n";
print "   & categories : $dstdir/categories/*\n";

open( IDX, "> $dstdir/index.txt");
open( ALL, "> $dstdir/alltags.txt");
foreach (sort keys %ctree) {
	my $key = $_;
	my $fold = 0;
	my %st = %{$ctree{$key}};
	my @tmp = keys %st;
	my @sub0=();
	my @sub1=();
#	print " ===== ".$key." ===== \n";
        if ($#tmp > 0) {
	  print ALL "++++ $key |\n";
	  $fold=1;
	}
	print ALL "  * ".makelink($key)."\n";
	print IDX "  * ".makelink($key)."\n";

	foreach (sort keys %st) {
		my $subkey = $_;
		my @sub2=();
		my %mt = %{$ctree{$key}{$subkey}};
		print ALL "    * ".makelink($subkey)."\n";
		print IDX "    * ".makelink($subkey)."\n";
		push @sub0, "$subkey";
		foreach (sort keys %mt) {
			my $ssc = $_;
			print ALL "      * ".makelink($_)."\n";
			print IDX "      * ".makelink($_)."\n";
			push @sub1, "$ssc";
			push @sub2, "$ssc";
			createcat($ssc,$subkey,(),());
		}
		createcat($subkey,$key,(),@sub2);
	}
	createcat($key,'',@sub0,@sub1);
	if ($fold) { print ALL "++++\n"; }
}
close(IDX);
close(ALL);
print "done.\n\n";

print "###########################\n";

if ($debug & 64 || !$renamedups) { 
	print "listing duplicates:\n";
	foreach (@duplicates) { print "+ ",$_,"\n"; } 
}

print "\n";
print "REPORT:\n";
print "-------\n";
printf " total entries: %5i\n", $entries;
printf " wiki Files:    %5i\n", $files;
printf " Duplicates:    %5i\n", $dups;
print "\n";
printf " Categories:    %5i\n", $catcount;
